--[[ Libs ]]--
AutoLog = LibStub("AceAddon-3.0"):NewAddon("AutoLog", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("AutoLog")
local bzone = LibStub("LibBabble-Zone-3.0")
local BZ = bzone:GetUnstrictLookupTable()

--[[ Normal mode raid list ]]--
local raid_list = {
	["ULD"] = BZ["Ulduar"],
	["NAX"] = BZ["Naxxramas"],
	["EYE"] = BZ["The Eye of Eternity"],
	["OBS"] = BZ["The Obsidian Sanctum"],
	["VOA"] = BZ["Vault of Archavon"],
	["TOC"] = BZ["Trial of the Crusader"],
	["ONY"] = BZ["Onyxia's Lair"],
	["ICC"] = BZ["Icecrown Citadel"],
	["RUS"] = BZ["The Ruby Sanctum"],
	["BWD"] = BZ["Blackwing Descent"],
	["BTW"] = BZ["The Bastion of Twilight"],
	["TFW"] = BZ["Throne of the Four Winds"],
	["BAH"] = BZ["Baradin Hold"],
}

--[[ Heroic mode raid list ]]--
local raid_heroic = {
	["TOC"] = BZ["Trial of the Crusader"],
	["ICC"] = BZ["Icecrown Citadel"],
	["RUS"] = BZ["The Ruby Sanctum"],
	["BWD"] = BZ["Blackwing Descent"],
	["BTW"] = BZ["The Bastion of Twilight"],
	["TFW"] = BZ["Throne of the Four Winds"],
}

--[[ Option table ]]--
local options = {
	name = "AutoLog",
	handler = AutoLog,
	type = 'group',
	args = {
		enable = { -- Global enable key
			type = "toggle",
			name = L["Enable auto logging"],
			desc = L["If unchecked, AutoLog will never enable Combat Logging"],
			get = "IsALenabled",
			set = "ToggleEnable",
			width = "double",
			order = 0,
		},
		dungeons = { -- Heroic dungeons
			type = "toggle",
			name = L["Log 5 player heroic instances"],
			set = "SetDungeons",
			get = "GetDungeons",
			order = 1,
			width = "full",
		},
		curLogging = { -- Logging indicator
			type = "toggle",
			name = L["Logging state"],
			disabled = true,
			desc = L["Checked if combat logging is currently on"],
			get = "IsCurrentlyLogging",
			set = "setCurrLog",
			order = 2,
		},
		raids10 = { -- 10 Normal raids select
			type = "multiselect",
			name = L["10 Players Raids"],
			desc = L["10 Players raid instances where you want to do a combat log"],
			values = raid_list,
			tristate = false,
			set = "SetRaid10",
			get = "GetRaid10",
			order = 3,
		},
		raids25 = { -- 25 Normal raids select
			type = "multiselect",
			name = L["25 Players Raids"],
			desc = L["25 Players raid instances where you want to do a combat log"],
			values = raid_list,
			tristate = false,
			set = "SetRaid25",
			get = "GetRaid25",
			order = 4,
		},
		raids10h = { -- 10 Heroic raids select
			type = "multiselect",
			name = L["10 Players Heroic Raids"],
			desc = L["10 Players heroic raid instances where you want to do a combat log"],
			values = raid_heroic,
			tristate = false,
			set = "SetRaid10h",
			get = "GetRaid10h",
			order = 5,
		},
		raids25h = { -- 25 Heroic raids select
			type = "multiselect",
			name = L["25 Players Heroic Raids"],
			desc = L["25 Players heroic raid instances where you want to do a combat log"],
			values = raid_heroic,
			tristate = false,
			set = "SetRaid25h",
			get = "GetRaid25h",
			order = 6,
		},
	},
}

--[[ Default DB ]]--
local defaults = {
	char = {
		enable = false,
		curLogging = false,
		raids10 = {
			["ULD"] = false,
			["NAX"] = false,
			["EYE"] = false,
			["OBS"] = false,
			["VOA"] = false,
			["TOC"] = false,
			["ONY"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
			["BAH"] = false,
		},
		raids25 = {
			["ULD"] = false,
			["NAX"] = false,
			["EYE"] = false,
			["OBS"] = false,
			["VOA"] = false,
			["TOC"] = false,
			["ONY"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
			["BAH"] = false,
		},
		raids10h = {
			["TOC"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
		},
		raids25h = {
			["TOC"] = false,
			["ICC"] = false,
			["RUS"] = false,
			["BWD"] = false,
			["BTW"] = false,
			["TFW"] = false,
		},
	},
}

--[[ Init function : Creates DB / chat commands ]]--
function AutoLog:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AutoLog", options)
	self.db = LibStub("AceDB-3.0"):New("AutoLog_DB", defaults)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AutoLog", "AutoLog v" .. GetAddOnMetadata("AutoLog","Version"))
	self:RegisterChatCommand("autolog", "ChatCommand")
	self:RegisterChatCommand("al", "ChatCommand")
end

--[[ Enable function : Register event ]]--
function AutoLog:OnEnable()
	self:Print("Version " .. GetAddOnMetadata("AutoLog","Version") .. L[" loaded."])
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	AutoLog:CheckLog()
end

--[[ Disable function ... should not be called, but must exist ]]--
function AutoLog:OnDisable()
	self:Print(L["AutoLog Disabled"])
	self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
end

--[[ Event handler ]]--
function AutoLog:ZONE_CHANGED_NEW_AREA()
	AutoLog:CheckLog()
end

--[[ GetKey function :
--		- Returns the key name of the raid, if it exists, false otherwise ]]--
function AutoLog:GetKey(table,area)
	for k, v in pairs (table) do
		--self:Print(k .. " : " .. v)
		if area == strlower(v) then
			return k
		end
	end
	return false
end

--[[ CheckLog function :
--		- Called on event / option changes
--		- Checks wether combat logging should be enabled (or not) ]]--
function AutoLog:CheckLog()

	local area = GetRealZoneText()
	local difficulty = GetInstanceDifficulty()
	local isLogging = LoggingCombat()
	local isDungeon, raidKey
	local goLog = false
	
	--self:Print("Current area : " .. area)
	if not self.db.char.enable then
	
		-- Add-on is disabled
		return
		
	end

	--fix to prevent an error occuring if area is nil
	if not area then
	
		return
		
	else
	
		area = strlower(area)
		
	end
	
	-- Here is just a "switch" between 4 difficulty levels
	if difficulty == 1 then
	
		-- 10 Normal
		raidKey = AutoLog:GetKey(raid_list,area)
		
		if raidKey then
		
			-- Known zone
			goLog = self.db.char.raids10[raidKey]
			
		end
		
	elseif difficulty == 2 then
	
		-- 25 Normal/ 5 Player Heroic
		raidKey = AutoLog:GetKey(raid_list,area)
		isDungeon = (select(2, GetInstanceInfo()) == "party")
		
		if ((self.db.char.dungeons) and (isDungeon)) then
		
			goLog = true
			
		elseif raidKey then
		
			-- Known zone
			goLog = self.db.char.raids25[raidKey]
		end
		
	elseif difficulty == 3 then
	
		-- 10 Heroic
		raidKey = AutoLog:GetKey(raid_heroic,area)
		
		if raidKey then
		
			-- Known zone
			goLog = self.db.char.raids10h[raidKey]
			
		end
		
	elseif difficulty == 4 then
	
		-- 25 Heroic
		raidKey = AutoLog:GetKey(raid_heroic,area)
		
		if raidKey then
		
			-- Known zone
			goLog = self.db.char.raids25h[raidKey]
			
		end
		
	end

	if goLog and not isLogging then
	
		self:Print(L["Combat logging turned on"])
		
	elseif isLogging and not goLog then
	
		self:Print(L["Combat logging turned off"])
		
	end
	
	self.db.char.curLogging = LoggingCombat(goLog)
	
end

function AutoLog:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(AutoLog, "autolog", "AutoLog", input)
	end
end

function AutoLog:SetCurrLog(info,newValue)
	self.db.char.curLogging = newValue
end

function AutoLog:IsCurrentlyLogging(info)
	self.db.char.curLogging = LoggingCombat()
	return self.db.char.curLogging
end

function AutoLog:IsALenabled(info)
	return self.db.char.enable
end

function AutoLog:ToggleEnable(info, newValue)
	self.db.char.enable = newValue
	AutoLog:CheckLog()
end

function AutoLog:GetRaid10(info,raid)
	return self.db.char.raids10[raid]
end

function AutoLog:SetRaid10(info,raid,val)
	self.db.char.raids10[raid] = val
	AutoLog:CheckLog()
end

function AutoLog:GetRaid25(info,raid)
	return self.db.char.raids25[raid]
end

function AutoLog:SetRaid25(info,raid,val)
	self.db.char.raids25[raid] = val
	AutoLog:CheckLog()
end

function AutoLog:GetRaid10h(info,raid)
	return self.db.char.raids10h[raid]
end

function AutoLog:SetRaid10h(info,raid,val)
	self.db.char.raids10h[raid] = val
	AutoLog:CheckLog()
end

function AutoLog:GetRaid25h(info,raid)
	return self.db.char.raids25h[raid]
end

function AutoLog:SetRaid25h(info,raid,val)
	self.db.char.raids25h[raid] = val
	AutoLog:CheckLog()
end

function AutoLog:GetDungeons()
	return self.db.char.dungeons
end

function AutoLog:SetDungeons(info, val)
	self.db.char.dungeons = val
	AutoLog:CheckLog()
end